#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "graph.h"

#ifndef NSIZE 
#define NSIZE 10
#endif

graph_t* allocate_graph(void) 
{
   graph_t *g = (graph_t*) malloc(sizeof(graph_t));
   assert(g != NULL);
   g->edges = NULL;
   g->length = 0;
   g->size = 0;
   /* or we can call calloc */
   return g;
}

void free_graph(graph_t **g) 
{
   assert(g != NULL && *g != NULL);
   if ((*g)->size > 0) {
      free((*g)->edges);
   }
   free(*g);
   *g = NULL;
}

graph_t* enlarge_graph(graph_t *g) 
{
   assert(g != NULL);
   int n = g->size == 0 ? NSIZE : g->size * 2; /* double the memory */

   edge_t *e = (edge_t*)malloc(n * sizeof(edge_t));
   assert(e != NULL);
   memcpy(e, g->edges, g->length * sizeof(edge_t));
   free(g->edges);
   g->edges = e;
   g->size = n;
   return g;
}

void print_graph(graph_t *g) 
{
   assert(g != NULL);
   fprintf(stderr, "Graph has %d edges and %d edges are allocated\n", g->length, g->size);
   edge_t *e = g->edges;
   for(int i = 0; i < g->length; ++i, ++e) {
      printf("%d %d %d\n", e->from, e->to, e->cost);
   }
}
